"""Per-prefix data, mapping each prefix to a dict of locale:name.

Auto-generated file, do not edit by hand.
"""
from phonenumbers.util import u

# Copyright (C) 2011-2017 The Libphonenumber Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

data = {
 '8255':{'en': 'Gyeongnam', 'ko': u('\uacbd\ub0a8')},
 '8232':{'en': 'Incheon', 'ko': u('\uc778\ucc9c')},
 '8231':{'en': 'Gyeonggi', 'ko': u('\uacbd\uae30')},
 '8251':{'en': 'Busan', 'ko': u('\ubd80\uc0b0')},
 '8253':{'en': 'Daegu', 'ko': u('\ub300\uad6c')},
 '8252':{'en': 'Ulsan', 'ko': u('\uc6b8\uc0b0')},
 '8210':{'en': 'Mobile prefix, should not be geocoded.'},
 '1650960':{'en': 'Mountain View, CA'},
 '1201':{'de': 'New Jersey', 'en': 'NJ'},
 '8264':{'en': 'Jeju'},
 '542214':{'en': 'La Plata'},
 '1989':{'en': 'MA'},
 '1212812':{'en': 'New York, NY'},
 '1650':{'de': 'Kalifornien', 'en': 'CA'},
 '1212':{'en': 'NY'},
 '8254':{'en': 'Gyeongbuk', 'ko': u('\uacbd\ubd81')},
 '8233':{'en': 'Gangwon', 'ko': u('\uac15\uc6d0')},
 '8242':{'en': 'Daejeon', 'ko': u('\ub300\uc804')},
 '8243':{'en': 'Chungbuk', 'ko': u('\ucda9\ubd81')},
 '8241':{'en': 'Chungnam', 'ko': u('\ucda9\ub0a8')},
 '8261':{'en': 'Jeonnam', 'ko': u('\uc804\ub0a8')},
 '8262':{'en': 'Gwangju', 'ko': u('\uad11\uc8fc')},
 '8263':{'en': 'Jeonbuk', 'ko': u('\uc804\ubd81')},
 '1617423':{'en': 'Boston, MA'},
 '822':{'en': 'Seoul', 'ko': u('\uc11c\uc6b8')},
}
